@@ -111,8 +111,6 @@ enum {
 	PROP_LOGO_RIGHT_PADDING,
 };
 
-static void gnome_about_instance_init (GnomeAbout *about);
-static void gnome_about_class_init (GnomeAboutClass *klass);
 static void gnome_about_finalize (GObject *object);
 static void gnome_about_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *pspec);
 static void gnome_about_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec);
@@ -121,7 +119,7 @@ static gboolean gnome_about_area_expose (GtkWidget *area, GdkEventExpose *event,
 static gboolean gnome_about_area_button_press (GtkWidget *area, GdkEventButton *button, gpointer data);
 
 GNOME_CLASS_BOILERPLATE (GnomeAbout, gnome_about,
-			 GtkDialog, gtk_dialog, GTK_TYPE_DIALOG)
+			 GtkDialog, GTK_TYPE_DIALOG)
 
 static void
 gnome_about_instance_init (GnomeAbout *about)
@@ -1167,14 +1165,33 @@ gnome_about_new (const gchar  *name,
 		 GdkPixbuf    *logo_pixbuf)
 {
 	GnomeAbout *about;
-	GValueArray *authors_array;
-	GValueArray *documenters_array;
-	static GdkPixbuf *background_pixbuf = NULL;
-	gint i;
 	
 	g_return_val_if_fail (authors != NULL, NULL);
 	
 	about = g_object_new (GNOME_TYPE_ABOUT, NULL);
+        gnome_about_construct(about, 
+			      name, version, copyright, 
+			      comments, authors, documenters, 
+			      translator_credits, logo_pixbuf);
+
+	return GTK_WIDGET(about);
+}
+
+void
+gnome_about_construct (GnomeAbout *about,
+		       const gchar  *name,
+		       const gchar  *version,
+		       const gchar  *copyright,
+		       const gchar  *comments,
+		       const gchar **authors,
+		       const gchar **documenters,
+		       const gchar  *translator_credits,
+		       GdkPixbuf    *logo_pixbuf)
+{
+	GValueArray *authors_array;
+	GValueArray *documenters_array;
+	static GdkPixbuf *background_pixbuf = NULL;
+	gint i;
 
 	authors_array = g_value_array_new (0);
 	
@@ -1226,8 +1243,6 @@ gnome_about_new (const gchar  *name,
 	}
 
 	gtk_window_set_resizable (GTK_WINDOW (about), FALSE);
-	
-	return GTK_WIDGET (about);
 }
 
 static void
@@ -1258,5 +1273,5 @@ gnome_about_finalize (GObject *object)
 	g_free (about->_priv);
 	about->_priv = NULL;
 
-	GNOME_CALL_PARENT_HANDLER (G_OBJECT_CLASS, finalize, (object));
+	GNOME_CALL_PARENT (G_OBJECT_CLASS, finalize, (object));
 }
