@@ -72,8 +72,6 @@ struct _GnomeAppPrivate
 };
 
 
-static void gnome_app_class_init  (GnomeAppClass *class);
-static void gnome_app_instance_init        (GnomeApp      *app);
 static void gnome_app_destroy     (GtkObject     *object);
 static void gnome_app_finalize    (GObject       *object);
 static void gnome_app_show        (GtkWidget     *widget);
@@ -94,7 +92,7 @@ static void   layout_changed      (GtkWidget     *widget,
 
 /* define _get_type and parent_class */
 GNOME_CLASS_BOILERPLATE (GnomeApp, gnome_app,
-			 GtkWindow, gtk_window, GTK_TYPE_WINDOW)
+			 GtkWindow, GTK_TYPE_WINDOW)
 
 static gchar *
 read_layout_config (GnomeApp *app)
@@ -151,7 +149,7 @@ gnome_app_class_init (GnomeAppClass *class)
 				      g_param_spec_string ("app_id",
 							   _("App ID"),
 							   _("The application ID string"),
-							   program_invocation_short_name,
+							   g_get_prgname (),
 							   (G_PARAM_READABLE |
 							    G_PARAM_WRITABLE)));
 }
@@ -192,17 +190,12 @@ gnome_app_get_property (GObject       *object,
 static void
 gnome_app_instance_init (GnomeApp *app)
 {
-	const char *str = NULL;
-	GValue value = { 0, };
-
+	const char *icons;
+		
 	app->_priv = NULL;
 	/* XXX: when there is some private stuff enable this
 	app->_priv = g_new0(GnomeAppPrivate, 1);
 	*/
-
-	app->name = NULL;
-	app->prefix = NULL;
-
 	app->accel_group = gtk_accel_group_new ();
 	gtk_window_add_accel_group (GTK_WINDOW (app), app->accel_group);
 	
@@ -210,6 +203,8 @@ gnome_app_instance_init (GnomeApp *app)
 	gtk_container_add (GTK_CONTAINER (app), app->vbox);
 
 	app->dock = bonobo_dock_new ();
+	app->layout = bonobo_dock_layout_new ();
+
 	gtk_box_pack_start (GTK_BOX (app->vbox), app->dock,
 			    TRUE, TRUE, 0);
 
@@ -218,21 +213,18 @@ gnome_app_instance_init (GnomeApp *app)
 			    GTK_SIGNAL_FUNC (layout_changed),
 			    (gpointer) app);
 
-	app->layout = bonobo_dock_layout_new ();
-
 	app->enable_layout_config = TRUE;
 
-	g_value_init (&value, G_TYPE_STRING);
-	g_object_get_property (G_OBJECT (gnome_program_get ()),
-			       LIBGNOMEUI_PARAM_DEFAULT_ICON, &value);
-	str = g_value_get_string (&value);
-	if (str != NULL) {
-		char **files = g_strsplit (str, ";", -1);
+	g_object_get (G_OBJECT (gnome_program_get ()),
+		      LIBGNOMEUI_PARAM_DEFAULT_ICON, &icons,
+		      NULL);
+
+	if (icons && *icons) {
+		char **files = g_strsplit (icons, ";", -1);
 		gnome_window_icon_set_from_file_list (GTK_WINDOW (app),
 						      (const char **)files);
 		g_strfreev (files);
 	}
-	g_value_unset (&value);
 }
 
 static void
@@ -259,14 +251,14 @@ gnome_app_show (GtkWidget *widget)
 		if (app->enable_layout_config)
 			write_layout_config (app, app->layout);
 
-		gtk_object_unref (GTK_OBJECT (app->layout));
+		g_object_unref (G_OBJECT (app->layout));
 		app->layout = NULL;
 	}
 			
 	gtk_widget_show (app->vbox);
 	gtk_widget_show (app->dock);
 
-	GNOME_CALL_PARENT_HANDLER (GTK_WIDGET_CLASS, show, (widget));
+	GNOME_CALL_PARENT (GTK_WIDGET_CLASS, show, (widget));
 }
 
 static void
@@ -284,7 +276,7 @@ layout_changed (GtkWidget *w, gpointer data)
 
 		layout = bonobo_dock_get_layout (BONOBO_DOCK (app->dock));
 		write_layout_config (app, layout);
-		gtk_object_unref (GTK_OBJECT (layout));
+		g_object_unref (G_OBJECT (layout));
 	}
 }
 
@@ -362,7 +354,7 @@ gnome_app_destroy (GtkObject *object)
 	app = GNOME_APP (object);
 
 	if (app->accel_group != NULL) {
-		gtk_accel_group_unref (app->accel_group);
+		g_object_unref (G_OBJECT (app->accel_group));
 		app->accel_group = NULL;
 	}
 
@@ -371,7 +363,7 @@ gnome_app_destroy (GtkObject *object)
 		app->layout = NULL;
 	}
 
-	GNOME_CALL_PARENT_HANDLER (GTK_OBJECT_CLASS, destroy, (object));
+	GNOME_CALL_PARENT (GTK_OBJECT_CLASS, destroy, (object));
 }
 
 static void
@@ -393,7 +385,7 @@ gnome_app_finalize (GObject *object)
 	g_free (app->_priv);
 	app->_priv = NULL;
 
-	GNOME_CALL_PARENT_HANDLER (G_OBJECT_CLASS, finalize, (object));
+	GNOME_CALL_PARENT (G_OBJECT_CLASS, finalize, (object));
 }
 
 /* Callback for an app's contents' parent_set signal.  We set the app->contents
