@@ -59,9 +59,6 @@ struct _GnomeDruidPageStandardPrivate
 };
 
 
-static void gnome_druid_page_standard_instance_init (GnomeDruidPageStandard      *druid_page_standard);
-static void gnome_druid_page_standard_class_init    (GnomeDruidPageStandardClass *class);
-
 static void gnome_druid_page_standard_get_property  (GObject                     *object,
 						     guint                        prop_id,
 						     GValue                      *value,
@@ -106,8 +103,7 @@ enum {
 };
 
 GNOME_CLASS_BOILERPLATE (GnomeDruidPageStandard, gnome_druid_page_standard,
-			 GnomeDruidPage, gnome_druid_page,
-			 GNOME_TYPE_DRUID_PAGE)
+			 GnomeDruidPage, GNOME_TYPE_DRUID_PAGE)
 
 static void
 gnome_druid_page_standard_class_init (GnomeDruidPageStandardClass *class)
@@ -368,7 +364,7 @@ gnome_druid_page_standard_get_property (GObject    *object,
 		break;
 	}
 
-	GNOME_CALL_PARENT_HANDLER (G_OBJECT_CLASS, get_property, (object, prop_id, value, pspec));
+	GNOME_CALL_PARENT (G_OBJECT_CLASS, get_property, (object, prop_id, value, pspec));
 }
 
 static void
@@ -465,7 +461,7 @@ gnome_druid_page_standard_set_property (GObject      *object,
 		break;
 	}
 
-	GNOME_CALL_PARENT_HANDLER (G_OBJECT_CLASS, set_property, (object, prop_id, value, pspec));
+	GNOME_CALL_PARENT (G_OBJECT_CLASS, set_property, (object, prop_id, value, pspec));
 }
 
 static void
@@ -479,7 +475,7 @@ gnome_druid_page_standard_finalize (GObject *object)
 	g_free (druid_page_standard->_priv);
 	druid_page_standard->_priv = NULL;
 
-	GNOME_CALL_PARENT_HANDLER (G_OBJECT_CLASS, finalize, (object));
+	GNOME_CALL_PARENT (G_OBJECT_CLASS, finalize, (object));
 }
 
 
@@ -498,7 +494,7 @@ gnome_druid_page_standard_destroy (GtkObject *object)
 		gdk_pixbuf_unref (druid_page_standard->top_watermark);
 	druid_page_standard->top_watermark = NULL;
 
-	GNOME_CALL_PARENT_HANDLER (GTK_OBJECT_CLASS, destroy, (object));
+	GNOME_CALL_PARENT (GTK_OBJECT_CLASS, destroy, (object));
 }
 
 static void
@@ -506,7 +502,7 @@ gnome_druid_page_standard_realize (GtkWidget *widget)
 {
 	GnomeDruidPageStandard *druid_page_standard = GNOME_DRUID_PAGE_STANDARD (widget);
 
-	GNOME_CALL_PARENT_HANDLER (GTK_WIDGET_CLASS, realize, (widget));
+	GNOME_CALL_PARENT (GTK_WIDGET_CLASS, realize, (widget));
 	
 	gnome_druid_page_standard_set_color (druid_page_standard);
 }
@@ -517,7 +513,7 @@ gnome_druid_page_standard_style_set (GtkWidget *widget,
 {
 	GnomeDruidPageStandard *druid_page_standard = GNOME_DRUID_PAGE_STANDARD (widget);
 
-	GNOME_CALL_PARENT_HANDLER (GTK_WIDGET_CLASS, style_set, (widget, old_style));
+	GNOME_CALL_PARENT (GTK_WIDGET_CLASS, style_set, (widget, old_style));
 	
 	gnome_druid_page_standard_set_color (druid_page_standard);
 }
@@ -528,7 +524,8 @@ gnome_druid_page_standard_prepare (GnomeDruidPage *page,
 {
 	gnome_druid_set_buttons_sensitive (GNOME_DRUID (druid), TRUE, TRUE, TRUE, TRUE);
 	gnome_druid_set_show_finish (GNOME_DRUID (druid), FALSE);
-	gtk_widget_grab_default (GNOME_DRUID (druid)->next);
+	if (GTK_IS_WINDOW (gtk_widget_get_toplevel (druid)))
+		gtk_widget_grab_default (GNOME_DRUID (druid)->next);
 }
 
 GtkWidget *
@@ -786,7 +783,7 @@ static void
 gnome_druid_page_standard_size_allocate (GtkWidget *widget,
 					 GtkAllocation *allocation)
 {
-	GNOME_CALL_PARENT_HANDLER (GTK_WIDGET_CLASS, size_allocate,
+	GNOME_CALL_PARENT (GTK_WIDGET_CLASS, size_allocate,
 				   (widget, allocation));
 	gnome_druid_page_standard_layout_setup (GNOME_DRUID_PAGE_STANDARD (widget));
 }
